/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __BOARD_H
#define __BOARD_H

#include "game.h"
#include "point.h"
#include "spline.h"
#include "sprites.h"
#include "effects.h"
#include <utility>
#include <vector>
#include "button.h"

using std::pair;
using std::vector;



// defined in effects.h
// enum TilePos { THIS, UPLEFT, UP, UPRIGHT, DOWNRIGHT, DOWN, DOWNLEFT };
const char *tileNames[] = { "THIS", "UPLEFT", "UP", "UPRIGHT", "DOWNRIGHT", "DOWN", "DOWNLEFT" };




class SplineInfo {
public:
	int tile, rotation, nr;
	bool opposite;
	SplineInfo() {}
	SplineInfo(int t, int r, int n, bool o) : tile(t), rotation(r), nr(n), opposite(o) {}
	bool operator!=(const SplineInfo &si) {
		if (tile!=si.tile) return true;
		if (rotation!=si.rotation) return true;
		if (nr!=si.nr) return true;
		if (opposite!=si.opposite) return true;
		return false;
	}
};



class PlayerStatus {
private:
public:
	int posX, posY;
	TilePos direction;
	int highlightColor;
	float rot;
	SDL_Surface *rotImg;
	bool finished, active;
	float totLength;
	vector<float> lengths;
	CompSpline *curSpline;
	SplineInfo splineInfo;
	int passedBlocks;
	int blocksAdded;
	bool winner;
	PlayerStatus() : posX(0), posY(0), direction(UPLEFT), highlightColor(0x00FF0000), rotImg(NULL), finished(false), active(false), curSpline(NULL), passedBlocks(0), blocksAdded(0), winner(false) {}
	PlayerStatus(int x, int y, TilePos dir, int c) : posX(x), posY(y), direction(dir), highlightColor(c), rotImg(NULL), finished(false), active(true), curSpline(NULL), passedBlocks(0), blocksAdded(0), winner(false) {}
	~PlayerStatus() { SDL_FreeSurface(rotImg); }
};



enum GameState { WAITING, ANIMATING, FADETILE, FINISHED, EMPTY };

#define GRIDWIDTH 9
#define GRIDHEIGHT 8



class ImageState {
public:
	int selectedTile;
	int tileRotation;
	int buttondown, buttonover;
	int buttondownTile;
	GameState state;
	SDL_Surface *curPointer;
};


class BoardGame : public Game2, private ImageState {
private:
	SDL_Surface *board;
	SDL_Surface *bkg;
	SDL_Surface *tile[10];
	SDL_Surface *endTile, *tileBkg;
	int numTiles[10];
	bool redrawLabels, redrawLabels2;
	SDL_Surface *tractor[4];
	SDL_Surface *pointer[3];
	IPoint tilePos[10];
	bool quit;
	PlayerStatus player[4];
	int curPlayer;
	SDL_Surface *curTile;
	pair<int,int> grid[GRIDWIDTH][GRIDHEIGHT]; // tileNr, tileRotation
	SDL_Surface *problemHighlight;
	bool tileIsOk;
	ImageState last;
	Uint32 lastTime;
	SDL_Rect highlightSize;
	IPoint curMousePos;
	IPoint curOffset;
	TTF_Font *font;
	SpriteEngine se;

	float animMaxLen;
	Uint32 animStartTime;
	Uint32 animModeStart;
	bool animFade;


	bool isPaused;
	Uint32 pauseStart;
	int result;


	void findPath(int tile, int rotation, TilePos dir, int &nr, bool &opposite);

	void setPointer(int n);

	Button buttons[3];
	int findButton(int x, int y);
	void buttonPress(int n);
	void updateButton(int n, int state);
	bool isPressable(int n);

	int findTile(int x, int y);





	bool coordsOk(int x, int y);
	int height(int x);
	void getCoords(int gridx, int gridy, Sint16 &x, Sint16 &y);
	void getDrawRect(int gridx, int gridy, SDL_Surface *img, SDL_Rect &r);
	void getDrawRect(int gridx, int gridy, const SDL_Rect &img, SDL_Rect &r);
	void addTileOffset(SDL_Rect &rect, int tile, int rot, bool middle=true);
	void getGridCoords(int x, int y, int &gridx, int &gridy);
	TilePos getPos(int x, int y, int tilex, int tiley);
	bool isInsideTile(int x, int y, int tilex, int tiley);
	int getRotation(TilePos dir);
	void nextPos(int &gridx, int &gridy, TilePos dir);

	void mergeRects(SDL_Rect &main, const SDL_Rect &r2);
	bool rectsIntersect(const SDL_Rect &r1, const SDL_Rect &r2);
	bool intersect(const SDL_Rect &drawable, const SDL_Rect &r2, SDL_Rect &global, SDL_Rect &local);

	TilePos rotateDir(TilePos in, int n);
	TilePos oppositeDir(TilePos in);
	void rotateTile(int nr, int steps, TilePos *array);
	bool advance(int &gridx, int &gridy, TilePos &dir);

	bool testAdvance(int x, int y, TilePos dir);
	void tryPiece(int player, int tile, int rotation, TilePos problems[], int &problemPos);
	bool tryPiece(int player, int tile, int rotation);
	void validate();
	void newTurn();

	void grayTile(int n);
	void mergeEndTile();


	void updateTractor(float pos, int player, SDL_Rect &out);
	SDL_Surface *createSplineTest(int x, int y, int w, int h);

public:
	BoardGame(SDL_Surface *screen, Uint32 time);
	~BoardGame();
	void handleEvent(SDL_Event &event);
	void update(Uint32 time, SDL_Surface *screen);
	bool finished();
	void redraw();
	void setUpdatedRect(void (*func)(const SDL_Rect &r));
	void startGame(int players);
	void pause(bool pause);
	void getRects(vector<SDL_Rect> &list);
	int getResult();
};


#endif
